@@ -76,6 +76,7 @@ def convert(
     replace_gathernd_to_pseudo_gathernd: Optional[bool] = False,
     replace_neg_to_pseudo_neg: Optional[bool] = False,
     replace_hardswish_to_pseudo_hardswish: Optional[bool] = False,
+    replace_erf_to_pseudo_erf: Optional[bool] = False,
     param_replacement_file: Optional[str] = '',
     mvn_epsilon: Optional[float] = 0.0000000001,
     non_verbose: Optional[bool] = False,
@@ -298,6 +299,9 @@ def convert(
     replace_hardswish_to_pseudo_hardswish: Optional[bool]
         Replace HardSwish with a pseudo HardSwish.
 
+    replace_erf_to_pseudo_erf: Optional[bool]
+        Replace Erf with a pseudo Erf.
+
     mvn_epsilon: Optional[float]
         For MeanVarianceNormalization.\n
         The number to be added to the variance to avoid division by zero when normalizing the value.\n
@@ -336,7 +340,7 @@ def convert(
     output_folder_path = fr'{output_folder_path}'
 
     # Input file existence check
-    if not os.path.exists(input_onnx_file_path):
+    if not os.path.exists(input_onnx_file_path) and not onnx_graph:
         print(
             f'{Color.RED}ERROR:{Color.RESET} ' +
             f'The specified *.onnx file does not exist. ' +
@@ -494,6 +498,7 @@ def convert(
         'replace_gathernd_to_pseudo_gathernd': replace_gathernd_to_pseudo_gathernd,
         'replace_neg_to_pseudo_neg': replace_neg_to_pseudo_neg,
         'replace_hardswish_to_pseudo_hardswish': replace_hardswish_to_pseudo_hardswish,
+        'replace_erf_to_pseudo_erf': replace_erf_to_pseudo_erf,
         'replacement_parameters': replacement_parameters,
         'mvn_epsilon': mvn_epsilon,
     }
@@ -1168,6 +1173,12 @@ def main():
         action='store_true',
         help='Replace HardSwish with a pseudo HardSwish.'
     )
+    parser.add_argument(
+        '-rerf',
+        '--replace_erf_to_pseudo_erf',
+        action='store_true',
+        help='Replace Erf with a pseudo Erf.'
+    )
     parser.add_argument(
         '-me',
         '--mvn_epsilon',
@@ -1251,6 +1262,7 @@ def main():
         replace_gathernd_to_pseudo_gathernd=args.replace_gathernd_to_pseudo_gathernd,
         replace_neg_to_pseudo_neg=args.replace_neg_to_pseudo_neg,
         replace_hardswish_to_pseudo_hardswish=args.replace_hardswish_to_pseudo_hardswish,
+        replace_erf_to_pseudo_erf=args.replace_erf_to_pseudo_erf,
         param_replacement_file=args.param_replacement_file,
         mvn_epsilon=args.mvn_epsilon,
         non_verbose=args.non_verbose,
